@@ -122,6 +122,9 @@ namespace Npgsql
 
 		// Used when we closed the connector due to an error, but are pretending it's open.
 		private bool _fakingOpen;
+        // Used when the connection is closed but an TransactionScope is still active
+        // (the actual close is postponed until the scope ends)
+	    private bool _postponingClose;
 
 		// Strong-typed ConnectionString values
 		private NpgsqlConnectionStringBuilder settings;
@@ -131,6 +134,9 @@ namespace Npgsql
 
         private NpgsqlPromotableSinglePhaseNotification promotable = null;
 
+        // A cached copy of the result of `settings.ConnectionString`
+        private string _connectionString;
+
 
 		/// <summary>
 		/// Initializes a new instance of the
@@ -151,18 +157,8 @@ namespace Npgsql
 		{
 			NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, CLASSNAME, "NpgsqlConnection()");
 
-			NpgsqlConnectionStringBuilder builder = cache[ConnectionString];
-			if (builder == null)
-			{
-				settings = new NpgsqlConnectionStringBuilder(ConnectionString);
-			}
-			else
-			{
-				settings = builder.Clone();
-			}
-
-			LogConnectionString();
-
+            LoadConnectionStringBuilder(ConnectionString);
+			
 			NoticeDelegate = new NoticeEventHandler(OnNotice);
 			NotificationDelegate = new NotificationEventHandler(OnNotification);
 
@@ -243,9 +239,14 @@ namespace Npgsql
         [Editor(typeof(ConnectionStringEditor), typeof(System.Drawing.Design.UITypeEditor))]
 #endif
 
-		public override String ConnectionString
-		{
-			get { return settings.ConnectionString; }
+        public override String ConnectionString
+        {
+            get
+            {
+                if (string.IsNullOrEmpty(_connectionString))
+                    RefreshConnectionString();
+                return settings.ConnectionString;
+            }
 			set
 			{
 				// Connection string is used as the key to the connector.  Because of this,
@@ -261,7 +262,7 @@ namespace Npgsql
 				{
 					settings = builder.Clone();
 				}
-				LogConnectionString();
+                LoadConnectionStringBuilder(value);
 			}
 		}
 
@@ -523,6 +524,11 @@ namespace Npgsql
 		/// </summary>
 		public override void Open()
 		{
+            // If we're postponing a close (see doc on this variable), the connection is already
+            // open and can be silently reused
+		    if (_postponingClose)
+		        return;
+
 			CheckConnectionClosed();
 
 			NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "Open");
@@ -539,8 +545,22 @@ namespace Npgsql
 											NpgsqlConnectionStringBuilder.GetKeyName(Keywords.UserName));
 			}
 
-			// Get a Connector.  The connector returned is guaranteed to be connected and ready to go.
-			connector = NpgsqlConnectorPool.ConnectorPoolMgr.RequestConnector(this);
+			// Get a Connector, either from the pool or creating one ourselves.
+            if (Pooling)
+            {
+                connector = NpgsqlConnectorPool.ConnectorPoolMgr.RequestConnector(this);                
+            }
+            else
+            {
+                connector = new NpgsqlConnector(this);
+
+                connector.ProvideClientCertificatesCallback += ProvideClientCertificatesCallbackDelegate;
+                connector.CertificateSelectionCallback += CertificateSelectionCallbackDelegate;
+                connector.CertificateValidationCallback += CertificateValidationCallbackDelegate;
+                connector.PrivateKeySelectionCallback += PrivateKeySelectionCallbackDelegate;
+
+                connector.Open();
+            }
 
 			connector.Notice += NoticeDelegate;
 			connector.Notification += NotificationDelegate;
@@ -554,6 +574,9 @@ namespace Npgsql
 			{
 				Promotable.Enlist(Transaction.Current);
 			}
+
+            this.OnStateChange (new StateChangeEventArgs(ConnectionState.Closed, ConnectionState.Open));
+
 		}
 
 		/// <summary>
@@ -581,7 +604,10 @@ namespace Npgsql
 
 			Close();
 
-			settings[Keywords.Database] = dbName;
+            // Mutating the current `settings` object would invalidate the cached instance, so work on a copy instead.
+            settings = settings.Clone();
+            settings[Keywords.Database] = dbName;
+            _connectionString = null;
 
 			Open();
 		}
@@ -598,28 +624,70 @@ namespace Npgsql
 		public override void Close()
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "Close");
-            
-			if (connector != null)
-			{
-				Promotable.Prepare();
-                // clear the way for another promotable transaction
-                promotable = null;
 
-				connector.Notification -= NotificationDelegate;
-				connector.Notice -= NoticeDelegate;
+		    if (connector == null)
+		        return;
 
-				if (SyncNotification)
-				{
-					connector.RemoveNotificationThread();
-				}
+		    if (promotable != null && promotable.InLocalTransaction)
+		    {
+		        _postponingClose = true;
+		        return;
+		    }
 
-				NpgsqlConnectorPool.ConnectorPoolMgr.ReleaseConnector(this, connector);
+		    ReallyClose();
+        }
 
+        private void ReallyClose()
+        {
+            NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "ReallyClose");
+            _postponingClose = false;
 
-				connector = null;
-			
-			}
-		}
+            // clear the way for another promotable transaction
+            promotable = null;
+
+            connector.Notification -= NotificationDelegate;
+            connector.Notice -= NoticeDelegate;
+
+            if (SyncNotification)
+            {
+                connector.RemoveNotificationThread();
+            }
+
+            if (Pooling)
+            {
+                NpgsqlConnectorPool.ConnectorPoolMgr.ReleaseConnector(this, connector);                
+            }
+            else
+            {
+                Connector.ProvideClientCertificatesCallback -= ProvideClientCertificatesCallbackDelegate;
+                Connector.CertificateSelectionCallback -= CertificateSelectionCallbackDelegate;
+                Connector.CertificateValidationCallback -= CertificateValidationCallbackDelegate;
+                Connector.PrivateKeySelectionCallback -= PrivateKeySelectionCallbackDelegate;
+
+                if (Connector.Transaction != null)
+                {
+                    Connector.Transaction.Cancel();
+                }
+
+                Connector.Close();
+            }
+
+            connector = null;
+
+            this.OnStateChange (new StateChangeEventArgs(ConnectionState.Open, ConnectionState.Closed));
+
+        }
+
+        /// <summary>
+        /// When a connection is closed within an enclosing TransactionScope and the transaction
+        /// hasn't been promoted, we defer the actual closing until the scope ends.
+        /// </summary>
+        internal void PromotableLocalTransactionEnded()
+        {
+            NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "PromotableLocalTransactionEnded");
+            if (_postponingClose)
+                ReallyClose();
+        }
 
 		/// <summary>
 		/// Creates and returns a <see cref="System.Data.Common.DbCommand">DbCommand</see>
@@ -653,25 +721,17 @@ namespace Npgsql
 		/// <b>false</b> when being called from the finalizer.</param>
 		protected override void Dispose(bool disposing)
 		{
-			if (!disposed)
-			{
-				if (disposing)
-				{
-					NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "Dispose");
-					Close();
-				}
-				else
-				{
-					if (FullState != ConnectionState.Closed)
-					{
-						NpgsqlEventLog.LogMsg(resman, "Log_ConnectionLeaking", LogLevel.Debug);
-                        NpgsqlConnectorPool.ConnectorPoolMgr.FixPoolCountBecauseOfConnectionDisposeFalse(this);
-					}
-				}
+		    if (disposed)
+		        return;
 
-				base.Dispose(disposing);
-				disposed = true;
+			if (disposing)
+			{
+				NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "Dispose");
+				Close();
 			}
+
+			base.Dispose(disposing);
+			disposed = true;
 		}
 
 		/// <summary>
@@ -722,6 +782,14 @@ namespace Npgsql
 			}
 		}
 
+        /// <summary>
+        /// Returns a copy of the NpgsqlConnectionStringBuilder that contains the parsed connection string values.
+        /// </summary>
+        internal NpgsqlConnectionStringBuilder CopyConnectionStringBuilder()
+        {
+            return settings.Clone();
+        }
+
 		/// <summary>
 		/// The connector object connected to the backend.
 		/// </summary>
@@ -761,7 +829,7 @@ namespace Npgsql
 		/// </summary>
 		internal byte[] Password
 		{
-			get { return settings.Password; }
+			get { return settings.PasswordAsByteArray; }
 		}
 
 		/// <summary>
@@ -871,12 +939,40 @@ namespace Npgsql
 		/// </summary>
 		private void LogConnectionString()
 		{
+            if (LogLevel.Debug >= NpgsqlEventLog.Level)
+                return;
+
 			foreach (string key in settings.Keys)
 			{
 				NpgsqlEventLog.LogMsg(resman, "Log_ConnectionStringValues", LogLevel.Debug, key, settings[key]);
 			}
 		}
 
+        /// <summary>
+        /// Sets the `settings` ConnectionStringBuilder based on the given `connectionString`
+        /// </summary>
+        /// <param name="connectionString">The connection string to load the builder from</param>
+        private void LoadConnectionStringBuilder(string connectionString)
+        {
+	        settings = cache[connectionString];
+	        if (settings == null)
+	        {
+		        settings = new NpgsqlConnectionStringBuilder(connectionString);
+		        cache[connectionString] = settings;
+	        }
+
+	        RefreshConnectionString();
+            LogConnectionString();
+        }
+
+        /// <summary>
+        /// Refresh the cached _connectionString whenever the builder settings change
+        /// </summary>
+        private void RefreshConnectionString()
+        {
+            _connectionString = settings.ConnectionString;
+        }
+
 		private void CheckConnectionOpen()
 		{
 			if (disposed)
@@ -900,7 +996,7 @@ namespace Npgsql
 				_fakingOpen = false;
 			}
 
-			if (connector == null)
+			if (_postponingClose || connector == null)
 			{
 				throw new InvalidOperationException(resman.GetString("Exception_ConnNotOpen"));
 			}
@@ -966,8 +1062,9 @@ namespace Npgsql
 				case "DataSourceInformation":
 					return NpgsqlSchema.GetDataSourceInformation();
 				case "DataTypes":
-				case "ReservedWords":
-					throw new NotSupportedException();
+                    throw new NotSupportedException();
+                case "ReservedWords":
+			        return NpgsqlSchema.GetReservedWords();
 				// custom collections for npgsql
 				case "Databases":
 					return new NpgsqlSchema(new NpgsqlConnection(ConnectionString)).GetDatabases(restrictions);
@@ -983,6 +1080,8 @@ namespace Npgsql
                     return new NpgsqlSchema(new NpgsqlConnection(ConnectionString)).GetIndexes(restrictions);
                 case "IndexColumns":
                     return new NpgsqlSchema(new NpgsqlConnection(ConnectionString)).GetIndexColumns(restrictions);
+                case "ForeignKeys":
+                    return new NpgsqlSchema(new NpgsqlConnection(ConnectionString)).GetForeignKeys(restrictions);
 				default:
 					throw new NotSupportedException();
 			}
