@@ -128,7 +128,11 @@ namespace Npgsql
                                 case -1:
                                     throw new EndOfStreamException();
                                 case 'Z':
-                                    context.Query(new NpgsqlCommand("UNLISTEN *", context));
+                                    //context.Query(new NpgsqlCommand("UNLISTEN *", context));
+                                    using(NpgsqlCommand cmd = new NpgsqlCommand("UNLISTEN *", context))
+                                    {
+                                        context.Query(cmd);
+                                    }
                                     return;
                             }
                         }
@@ -311,7 +315,7 @@ namespace Npgsql
         ///
         internal void ProcessBackendResponses(NpgsqlConnector context)
         {
-            IterateThroughAllResponses(ProcessBackendResponsesEnum(context));
+            IterateThroughAllResponses(ProcessBackendResponsesEnum(context, false));
         }
 
         private static void IterateThroughAllResponses(IEnumerable<IServerResponseObject> ienum)
@@ -347,23 +351,24 @@ namespace Npgsql
         /// to handle backend requests.
         /// </summary>
         ///
-        internal IEnumerable<IServerResponseObject> ProcessBackendResponsesEnum(NpgsqlConnector context)
+        internal IEnumerable<IServerResponseObject> ProcessBackendResponsesEnum(NpgsqlConnector context,
+            bool cancelRequestCalled)
         {
             try
             {
             // Process commandTimeout behavior.
 
             if ((context.Mediator.CommandTimeout > 0) &&
-                (!context.Socket.Poll(1000000*context.Mediator.CommandTimeout, SelectMode.SelectRead)))
+				    (!CheckForContextSocketAvailability(context, SelectMode.SelectRead)))
             {
                 // If timeout occurs when establishing the session with server then
                 // throw an exception instead of trying to cancel query. This helps to prevent loop as CancelRequest will also try to stablish a connection and sends commands.
-                if (!((this is NpgsqlStartupState || this is NpgsqlConnectedState || context.CancelRequestCalled)))
+                if (!((this is NpgsqlStartupState || this is NpgsqlConnectedState || cancelRequestCalled)))
                 {
                     try
                     {
                         context.CancelRequest();
-                        foreach (IServerResponseObject obj in ProcessBackendResponsesEnum(context))
+                        foreach (IServerResponseObject obj in ProcessBackendResponsesEnum(context, true))
                         {
                             if (obj is IDisposable)
                             {
@@ -407,6 +412,40 @@ namespace Npgsql
                 
         }
 
+
+		/// <summary>
+		/// Checks for context socket availability.
+		/// Socket.Poll supports integer as microseconds parameter.
+		/// This limits the usable command timeout value
+		/// to 2,147 seconds: (2,147 x 1,000,000 < max_int).
+		/// In order to bypass this limit, the availability of
+		/// the socket is checked in 2,147 seconds cycles
+		/// </summary>
+		/// <returns><c>true</c>, if for context socket availability was checked, <c>false</c> otherwise.</returns>
+		/// <param name="context">Context.</param>
+		/// <param name="selectMode">Select mode.</param>
+		internal bool CheckForContextSocketAvailability (NpgsqlConnector context, SelectMode selectMode)
+		{
+			/* Socket.Poll supports integer as microseconds parameter.
+			 * This limits the usable command timeout value
+			 * to 2,147 seconds: (2,147 x 1,000,000 < max_int).
+			 */
+			const int limitOfSeconds = 2147;
+			
+			bool socketPoolResponse = false;
+			int secondsToWait = context.Mediator.CommandTimeout;
+			
+			/* In order to bypass this limit, the availability of
+			 * the socket is checked in 2,147 seconds cycles
+			 */
+			while ((secondsToWait > limitOfSeconds) && (!socketPoolResponse)) {    //
+				socketPoolResponse = context.Socket.Poll (1000000 * limitOfSeconds, selectMode);
+				secondsToWait -= limitOfSeconds;
+			}
+			
+			return socketPoolResponse || context.Socket.Poll (1000000 * secondsToWait, selectMode);
+		}
+
         protected IEnumerable<IServerResponseObject> ProcessBackendResponses_Ver_2(NpgsqlConnector context)
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "ProcessBackendResponses");
