@@ -78,8 +78,7 @@ namespace Npgsql
 		private static readonly byte[] THRASH_CAN = new byte[THRASH_CAN_SIZE];
 		private static readonly Encoding ENCODING_UTF8 = Encoding.UTF8;
 
-
-
+        private static readonly string NULL_TERMINATOR_STRING = '\x00'.ToString();
 
 
 
@@ -348,15 +347,28 @@ namespace Npgsql
 		/// </summary>
 		public static void WriteString(String the_string, Stream network_stream)
 		{
+            
 			NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "WriteString");
 
 			NpgsqlEventLog.LogMsg(resman, "Log_StringWritten", LogLevel.Debug, the_string);
 
-			byte[] bytes = ENCODING_UTF8.GetBytes(the_string + '\x00');
+			byte[] bytes = ENCODING_UTF8.GetBytes(the_string + NULL_TERMINATOR_STRING);
 			
 			network_stream.Write(bytes, 0, bytes.Length);
 		}
 
+        /// <summary>
+        /// This method writes a set of bytes to the stream. It also enables logging of them.
+        /// </summary>
+        public static void WriteBytes(byte[] the_bytes, Stream network_stream)
+        {
+            NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "WriteBytes");
+            NpgsqlEventLog.LogMsg(resman, "Log_BytesWritten", LogLevel.Debug, the_bytes);
+
+            network_stream.Write(the_bytes, 0, the_bytes.Length);
+            network_stream.Write(new byte[1], 0, 1);
+        }
+
 		///<summary>
 		/// This method writes a C NULL terminated string limited in length to the
 		/// backend server.
@@ -534,18 +546,11 @@ namespace Npgsql
 		    return sb;
 		}
 
-
-
         internal static void LogStringWritten(string theString)
-
         {
-
             NpgsqlEventLog.LogMsg(resman, "Log_StringWritten", LogLevel.Debug, theString);
-
             
-
         }
-
     }
 
 	/// <summary>
