@@ -3,8 +3,13 @@ using System;
 using System.Collections.Generic;
 using System.Linq;
 using System.Text;
+#if ENTITIES6
+using System.Data.Entity.Core.Common.CommandTrees;
+using System.Data.Entity.Core.Metadata.Edm;
+#else
 using System.Data.Metadata.Edm;
 using System.Data.Common.CommandTrees;
+#endif
 using NpgsqlTypes;
 using System.Data;
 
@@ -135,6 +140,10 @@ namespace Npgsql.SqlGenerators
                 case PrimitiveTypeKind.Guid:
                 case PrimitiveTypeKind.String:
                     NpgsqlTypesHelper.TryGetNativeTypeInfo(GetDbType(_primitiveType), out typeInfo);
+                    // Escape syntax is needed for strings with escape values.
+                    // We don't check if there are escaped strings for performance reasons.
+                    // Check https://github.com/franciscojunior/Npgsql2/pull/10 for more info.
+                    sqlText.Append('E'); 
                     sqlText.Append(typeInfo.ConvertToBackend(_value, false));
                     break;
                 case PrimitiveTypeKind.Time:
