@@ -30,7 +30,6 @@ using Npgsql;
 using NpgsqlTypes;
 
 using NUnit.Framework;
-using NUnit.Core;
 
 namespace NpgsqlTests
 {
@@ -160,6 +159,7 @@ namespace NpgsqlTests
         }
         
         [Test]
+        [Ignore]
         public void DataAdapterUpdateReturnValue2()
         {
             
@@ -204,6 +204,7 @@ namespace NpgsqlTests
         }
 
         [Test]
+        [Ignore]
         public void FillAddWithKey()
         {
             DataSet ds = new DataSet();
@@ -350,6 +351,7 @@ namespace NpgsqlTests
         }
         
         [Test]
+        [Ignore]
         public void UpdateWithDataSet()
         {
             DoUpdateWithDataSet();
@@ -393,6 +395,7 @@ namespace NpgsqlTests
         }
         
         [Test]
+        [Ignore]
         public void InsertWithCommandBuilderCaseSensitive()
         {
             DoInsertWithCommandBuilderCaseSensitive();
@@ -471,6 +474,20 @@ namespace NpgsqlTests
                 Console.Out.WriteLine(dr["dauer"]);
             }
         }
+
+        [Test]
+        public void DbDataAdapterCommandAccess()
+        {
+
+            NpgsqlCommand command = new NpgsqlCommand();
+            command.CommandType = CommandType.Text;
+            command.CommandText = "SELECT CAST('1 hour' AS interval) AS dauer";
+            command.Connection = TheConnection;
+            NpgsqlDataAdapter da = new NpgsqlDataAdapter();
+            da.SelectCommand = command;
+            System.Data.Common.DbDataAdapter common = da;
+            Assert.IsNotNull(common.SelectCommand);
+        }
     }
     [TestFixture]
     public class DataAdapterTestsV2 : DataAdapterTests
